;**********************************************************************
;                                                                     *
;    Filename:      pb12.inc                                          *
;    Date:                                                            *
;    File Version:                                                    *
;                                                                     *
;    Author:        el@jap.hu                                         *
;                   http://jap.hu/                                    *
;**********************************************************************
;    Notes:
;
; 2400 baud serial PIC bus
; for interPIC communication
; btw. 1 MASTER, 1-15 SLAVE(s)
; for INT-less uc-s
;
; Needs TMR0 programmed to 1:4 fosc/4!
; Program PB0 as input, PA4 as output
;
;**********************************************************************
;HISTORY
;
;020 - first include for p12 family
;021 - 20010526 working
;022 - 20010527
;023 - 20010616 adjusted optimal timings
;**********************************************************************

#define DBIT PORTB, 3

#ifdef SLAVE
rxwgap		; wait for a gap at least tmingap long
		; exit at the end of the gap
		movlw .256-(tmaxsig/.64); max LOW
		movwf TMR0

		; wait for the end of the LOW signal
rxwgap0		movf TMR0, W
		btfsc STATUS, Z
		retlw 0x82 ; short-circuit error

		btfss RXBIT
		goto rxwgap0

		movlw .256-(tmingap/.64) ; test for the gap min
		movwf TMR0

rxwgap1		movf TMR0, W
		bz rxwgap2 ; min gap is over!
		btfsc RXBIT
		goto rxwgap1

		; gap is less than min, need to start over!
		retlw 0x83

rxwgap2		; still gap, wait for up to gap max-min
		movlw .256-((tmaxgap-tmingap)/.64)
		movwf TMR0

rxwgap3		movf TMR0, W
		btfsc STATUS, Z
		retlw 0x84 ; gap too long

		btfsc RXBIT
		goto rxwgap3
		retlw 0 ; OK, startbit present
#endif

rxwhead		; wait 4 a startbit then receive a byte
		; timeout is ~ 1 ms
		movlw 0xf0

rxwhead2	; same with custom timeout
		movwf TMR0
rxw0		movf TMR0, W
		bz rxtimeout
		btfsc RXBIT ; wait 4 a startbit (L)
		goto rxw0

rx		; receive a byte into bt
		; after the arrive of a startbit

		call dlyhalfbit ; half bit delay
		movlw 8
		movwf btcnt

rx0		call dlyhalfbit
		call dlyhalfbit
		bcf STATUS, C
		btfsc RXBIT
		bsf STATUS, C
		rrf bt, F
		decfsz btcnt, F
		goto rx0
		call dlyhalfbit
		call dlyhalfbit
		btfss RXBIT ; STOPbit must be H
		retlw 0x80 ; receive error
		call dlyhalfbit
		retlw 0 ; OK, byte is in bt
rxtimeout	retlw 0x81 ; timeout waiting

tx		;transmit the byte in W
		movwf bt
		movlw 8
		movwf btcnt

		;startbit
		movf txport_shadow, W
		movwf TXPORT

bsf DBIT
		call dlyhalfbit
		call dlyhalfbit
bcf DBIT

txcycle		rrf bt, F
		movlw TXMASK
		btfss STATUS, C
		clrw
		iorwf txport_shadow, W
		movwf TXPORT
		call dlyhalfbit
		call dlyhalfbit
		decfsz btcnt, F
		goto txcycle

		movlw TXMASK
		iorwf txport_shadow, W
		movwf TXPORT
		call dlyhalfbit
		call dlyhalfbit
		return

dlyhalfbit	movlw 0x44
		movwf dlcnt
dlycycle	decfsz dlcnt, F
		goto dlycycle
		return

#ifdef MASTER
silence		; delay at least 10 msec
		; (2 byte @ 1200 baud)
		movlw 0x18
silence2	movwf bfcnt
silcycle	call dlyhalfbit
		decfsz bfcnt, F
		goto silcycle
		return
#endif

SUMCALC		macro
		; calculate the sum of buffer
chksum_set	movlw buffer
		movwf FSR
		movlw 0x0f
		andwf byte0, W
		movwf bfcnt
		movlw 2
		addwf bfcnt, F
		clrw

chksum_set0	addwf INDF, W
		incf FSR, F
		decfsz bfcnt, F
		goto chksum_set0

		clrf chksum
		subwf chksum, F

		endm

SUMCHK		macro err_label
		; check the sum of buffer, set W & Z
chksum_chk	movlw buffer
		movwf FSR
		movlw 0x0f
		andwf byte0, W
		movwf bfcnt
		movlw 3
		addwf bfcnt, F
		clrw

chksum_chk0	addwf INDF, W
		incf FSR, F
		decfsz bfcnt, F
		goto chksum_chk0

		andlw 0xff
		btfss STATUS, Z
		goto err_label ; chksum error...

		endm

BUFTX		macro
txmsg		movlw buffer
		movwf FSR
		movlw 0x0f
		andwf byte0, W
		movwf bfcnt
		movlw 2
		addwf bfcnt, F

txmsg0		movf INDF, W
		call tx
		incf FSR, F
		decfsz bfcnt, F
		goto txmsg0
		movf chksum, W
		call tx
		endm

BUFRX		macro ver, err_label
		; receive a buffer, W=read timeout

if ver == 0
		call rxwgap
		andlw 0xff
		bnz err_label

		bsf DBIT
		call rx
		bcf DBIT
else
		call rxwhead
endif
		andlw 0xff
		bnz err_label
		;
		movf bt, W
		movwf byte0
		movwf chksum

		; receive response
		movlw command
		movwf FSR
		movlw 0x0f
		andwf byte0, W
		movwf bfcnt
		movlw 2		
		addwf bfcnt, F

rxmsg2		movlw .256-(tmaxdelay/.64)

		bsf DBIT
		call rxwhead2 ; receive timeout is maxdelay usec between bytes
		bcf DBIT
		andlw 0xff
		bnz err_label; error receiving

		movf bt, W
		movwf INDF
		addwf chksum, F
		incf FSR, F
		decfsz bfcnt, F
		goto rxmsg2
		; received buffer

		; check chksum
		movf chksum, W
		iorlw 0
		movlw 0x86
		bnz err_label ; chksum error

		; check DEVID
		movlw 0xf0
		andwf byte0, W
		movwf bt

		movlw DEVID
		subwf bt, W
;		btfss STATUS, Z
;		goto err_label ; not me...
		btfsc STATUS, Z
		goto rx_ok
		movlw 0x85
		goto err_label ; not me...
rx_ok		; all done

		endm

HANDLESTD	macro tx_label
		; handle standard commands
		movlw 0x5f
		subwf command, W
		btfsc STATUS, Z
		goto cmd_ping

		movlw 0x5e
		subwf command, W
		btfss STATUS, Z
		goto cmd_cont
cmd_cver	movlw 2
		movwf byte0
		movlw 0x60
		movwf command
		movlw PBUS_VER
		movwf data0
		movlw MAIN_VER
		movwf data0+1
		goto tx_label

cmd_ping	clrf byte0
		movlw 0x6f
		movwf command
		goto tx_label

cmd_cont	; done
		endm

